program Algoritmo6_18;
(*
 * Algoritmo 6.18. Justificacion de un texto
 * Titulo del libro: Una introduccion a la programacion.
 *                   Un enfoque algoritmico
 * Autores del libro: Jesus J. Garcia Molina
 *                    Francisco J. Montoya Dato
 *                    Jose L. Fernandez Aleman
 *                    Maria J. Majado Rosales
 * Fecha: 1/9/2005
 * Capitulo 6. Tablas
 * Fichero de entrada: dat6_18.txt
 *)

uses crt, unitmsc1;

const
   Espacio = ' ';
   SangradoMaximo = 15;
   LongMaxPal = 20;
   AnchoMinLinea = LongMaxPal + SangradoMaximo;
   AnchoMaxLinea = 80;

type
    LongitudPalabra = 1..LongMaxPal;
    Palabra = record
                  contenido : array [LongitudPalabra] of char;
		  longitud : LongitudPalabra
    end;
    PosEnLinea = 1..AnchoMaxLinea;
    LongitudLinea = 0..AnchoMaxLinea;
    Linea = record
                  longitud : LongitudLinea;
                  contenido : array [PosEnLinea] of char;
    end;
    LongSangrado = 0..SangradoMaximo;
    LimAnchoLinea = AnchoMinLinea..AnchoMaxLinea;
var
    sangrado : LongSangrado;
    anchoLinea : LimAnchoLinea;
    palabraActual : Palabra;
    lineaActual : Linea;
    FinDeSecuenciaPalabra : boolean;
    S : MSC1;
    MarcaParrafo : Palabra;

(* **************                                       ********************
 * Operaciones secuencia intermedia de palabras
 * **************                                       ********************)

procedure ObtenerPalabra;
var
   i : integer;
begin
     (* Ignora los espacios en blanco *)
     while (EA_MSC1(S) <> MSC1_MarcaFin) and
           ((EA_MSC1(S) = Espacio) or (EA_MSC1(S) = MSC1_MarcaFinLinea)) do
	   Avanzar_MSC1 (S);
     FinDeSecuenciaPalabra := EA_MSC1(S) = MSC1_MarcaFin;
     if not (EA_MSC1(S) = MSC1_MarcaFin) then begin
     (* Almacena en el campo palabraActual.contenido la siguiente palabra *)
          i := 0;
          repeat
                i := i + 1;
	        palabraActual.contenido[i] := EA_MSC1 (S);
	        Avanzar_MSC1 (S);
          until (EA_MSC1(S) = MSC1_MarcaFin) or ((EA_MSC1(S) = Espacio) or
                                                ((EA_MSC1(S) = MSC1_MarcaFinLinea)));
          palabraActual.longitud := i;
     end
end;

(* *********************                                       *************)
procedure ComenzarPalabra;
begin
     TratamientoInicial_MSC1(S);
     Cargar_fichero_MSC1(S,'dat6_18.txt');
     Comenzar_MSC1(S);
     ObtenerPalabra;
end;

(* *********************                                       *************)
procedure AvanzarPalabra;
begin
     ObtenerPalabra;
end;

(* **************                                       ********************
 * Operaciones relacionadas con el tratamiento de palabras
 * **************                                       ********************)
function Longitud (pal: Palabra) : LongitudPalabra;
begin
     Longitud := pal.longitud;
end;
(* *********************                                       *************)
function Igual (p1, p2 : Palabra) : boolean;
var
   long, i : LongitudPalabra;
   igualLong : boolean;
begin
     igualLong := p1.longitud = p2.longitud;
     if igualLong then begin
		long := p1.longitud;
                i := 1;
	        while (i <> long) and (p1.contenido[i] = p2.contenido[i]) do
                      i := i + 1;
		igualLong := (p1.contenido[i] = p2.contenido[i]);
     end;
     Igual := igualLong
end;


(* **************                                       ********************
 * Grupo de operaciones de tratamiento de lineas
 * **************                                       ********************)
procedure AnyadirCaracterALinea (car : char);
begin
     lineaActual.longitud := lineaActual.longitud + 1;
     lineaActual.contenido[lineaActual.longitud] := car
end;

(* *********************                                       *************)
procedure AnyadirPalabraALinea;
var
   i : LongitudPalabra;
begin
     for i := 1 to palabraActual.longitud do
         AnyadirCaracterALinea (palabraActual.contenido[i])
end;

(* *********************                                       *************)
procedure InicializarPropLinea (var longSangrado: LongSangrado;
                                var longLinea : LimAnchoLinea);
var
   sangradoLinea, anchoLinea : integer;
begin
     write('Introduzca el ancho de la linea (entre ', AnchoMinLinea,
           ' y ', AnchoMaxLinea, '): ');
     readln(anchoLinea);
     if anchoLinea < AnchoMinLinea then longLinea := AnchoMinLinea
     else if anchoLinea > AnchoMaxLinea then longLinea := AnchoMaxLinea
     else longLinea := anchoLinea;
     writeln;
     write('Introduzca el sangrado de la primera linea (entre 0',
           ' y ', SangradoMaximo, '): ');
     readln(sangradoLinea);
     if sangradoLinea < 0 then longSangrado := 0
     else if sangradoLinea > SangradoMaximo then longSangrado := SangradoMaximo
     else longSangrado := sangradoLinea;

     MarcaParrafo.longitud := 5;
     MarcaParrafo.contenido[1] := '/';
     MarcaParrafo.contenido[2] := 'P';
     MarcaParrafo.contenido[3] := 'A';
     MarcaParrafo.contenido[4] := 'R';
     MarcaParrafo.contenido[5] := '/';

end;

(* *********************                                       *************)
procedure SangrarLinea (longSangrado : integer);
var
   i : 1..SangradoMaximo;
begin
     for i := 1 to longSangrado do
         AnyadirCaracterALinea(Espacio)
end;

(* *********************                                       *************)
procedure InicializarLinea (longSangrado : integer);
begin
     lineaActual.longitud := 0;
     if longSangrado > 0 then SangrarLinea(longSangrado)
end;

(* *********************                                       *************)
function CabeEnLinea (espacioSolicitado: LongitudPalabra;
                       longLinea : integer) : boolean;
begin
     CabeEnLinea := espacioSolicitado <= (longLinea - lineaActual.longitud)
end;

(* *********************                                       *************)
procedure EscribirLinea (justificacion : boolean;
                         longLinea, longSangrado : integer);
     (* *********************                                       ********)
     procedure EscribirLineaJustificada (longLinea, longSangrado : integer);
     var
        numHuecos, blancosExtraMin : LongitudLinea;
        numHuecosAct, numHuecosMin : LongitudLinea;
        limInf, limSup, pos : PosEnLinea;
        (* *********************                                       *****)
        procedure EscribirEspacios (numEspacios : PosEnLinea);
        var
	   i : PosEnLinea;
        begin (* EscribirEspacios *)
             for i := 1 to numEspacios do
                 write(Espacio);
        end;  (* EscribirEspacios *)

     begin (* EscribirLineaJustificada
            * Calcular limite inferior y superior *)
          if lineaActual.contenido[1] = Espacio
              (* Si es primera linea de un parrafo *)
              then limInf := longSangrado + 1
          else limInf := 1;
          if lineaActual.contenido[lineaActual.longitud] = Espacio
             then limSup := lineaActual.longitud - 1
          else limSup := lineaActual.longitud;
          EscribirEspacios(limInf - 1);

          (* Calcular nmero de huecos *)
          numHuecos := 0;
          for pos := limInf to limSup do
              if lineaActual.contenido[pos] = Espacio then
                 numHuecos := numHuecos + 1;

          (* Calcular numero de espacios a insertar y numero de huecos
           * en los que se inserta ese numero de espacios, en el resto
           * se inserta uno mas *)
          if (numHuecos <> 0) then begin
             blancosExtraMin := (longLinea - limSup) div numHuecos;
             numHuecosMin := numHuecos -
                             (longLinea - limSup) mod numHuecos
          end;

          (* Recorrido para insertar espacios en los huecos *)
          numHuecosAct := 0;
          for pos := limInf to limSup do
              if (lineaActual.contenido[pos] = Espacio) and
                 (numHuecosAct < numHuecosMin) then begin
                 EscribirEspacios (blancosExtraMin + 1);
                 numHuecosAct := numHuecosAct + 1;
              end
              else if (lineaActual.contenido[pos] = Espacio) and
                      (numHuecosAct >= numHuecosMin)
                      then EscribirEspacios(blancosExtraMin+2)
              else if (lineaActual.contenido[pos] <> Espacio)
              then write(lineaActual.contenido[pos])
     end; (* EscribirLineaJustificada *)

     (* *********************                                       ********)
     procedure EscribirLineaSinJustificar;
     var
        i : PosEnLinea;
     begin (* EscribirLineaSinJustificar *)
          for i := 1 to lineaActual.longitud - 1 do
              write(lineaActual.contenido[i]);
          if lineaActual.contenido[lineaActual.longitud] <> Espacio then
             write(lineaActual.contenido[lineaActual.longitud])
     end;   (* EscribirLineaSinJustificar *)

begin (* EscribirLinea *)
     if justificacion then EscribirLineaJustificada(longLinea, longSangrado)
     else EscribirLineaSinJustificar;
     writeln
end;  (* EscribirLinea *)

begin (* Principal *)
     clrscr;
     writeln;
     writeln('*****');
     writeln('***** Algoritmo 6.18. Justificacion de un texto *****');
     writeln('*****');
     writeln;
     InicializarPropLinea(sangrado, anchoLinea);
     InicializarLinea(sangrado);
     writeln;
     write('El texto justificado es:');
     writeln;
     writeln;
     ComenzarPalabra;
     while not FinDeSecuenciaPalabra do begin
           if Igual(palabraActual, MarcaParrafo) then begin
              EscribirLinea(false, anchoLinea, sangrado);
              writeln;
              InicializarLinea(sangrado)
           end
           else begin
                if not CabeEnLinea(Longitud(palabraActual), anchoLinea) then begin
                       EscribirLinea(true, anchoLinea, sangrado);
                       InicializarLinea (0);
                end;
                AnyadirPalabraALinea;
                if CabeEnLinea(1, anchoLinea) then
                   AnyadirCaracterALinea(Espacio)
           end;
           AvanzarPalabra
     end;
     EscribirLinea(false, anchoLinea , sangrado);
     writeln;
     write('Pulse enter para continuar');
     readln

end.  (* Principal *)


