program Algoritmo7_14_15;
(*
 * Algoritmo 7.14. Algoritmo de ordenacion rapida de Hoare
 * Algoritmo 7.15. Accion de particionamiento para la accion de clasificacion rapida
 * Titulo del libro: Una introduccion a la programacion.
 *                   Un enfoque algoritmico
 * Autores del libro: Jesus J. Garcia Molina
 *                    Francisco J. Montoya Dato
 *                    Jose L. Fernandez Aleman
 *                    Maria J. Majado Rosales
 * Capitulo 7. Disenyo recursivo
 *)

(*
 * HOARE
 *
 * Implementa el algoritmo de clasificacion rapida de C. A. R. Hoare.
 * Solicita los datos para la generacion de los elementos del vector
 * (valores enteros): semilla aleatoria, numero de elementos que se
 * desean generar (con un maximo determinado por la constante MAX), y
 * valor maximo que se generara. Muestra el vector antes y despues de
 * la clasificacion, y comprueba que efectivamente esta ordenado tras
 * la llamada a la funcion de clasificacion.
 *)

uses crt;

const
     MAX = 3000;

type
    TipoBase = integer;
    indice = 1..MAX;
    TDatos = array [indice] of TipoBase;

var
   t : TDatos;
   max_real : indice;

procedure Comprueba(v : TDatos; mr : indice);
(*
 * Comprueba que los 'mr' primeros elementos del vector 'v' esten
 * efectivamente ordenados. En caso contrario imprime un mensaje
 * de error indicando la posicion 'i' donde hay una inversion, es
 * decir, donde v[i] > v[i+1].
 *)
var
   i : indice;
   error : boolean;
begin
     i := 1;
     error := false;
     while (i < mr) and not error do
		if v[i] > v[i+1] then begin
			write('<<< ERROR >>> (i=', i, ')');
			error := true
                end
		else
		    i := i + 1
end;

procedure InitTDatos(VAR v : TDatos; VAR mr : indice);
var
	i : indice;
	seed, maximo : integer;
begin
     write('Semilla aleatoria: ');
     readln(seed);
     RandSeed := seed;
     writeln;
     write('Elementos en el vector (minimo 1, maximo ', MAX, '): ');
     readln(mr);
     writeln;
     write('Generar valores entre cero y: ');
     readln(maximo);
     for i := 1 to mr do v[i] := Random(maximo+1)
end;

procedure MostrarTDatos(v : TDatos; mr : indice);
var
   i : indice;
begin
     i := 1;
     write(v[i]);
     while i <> mr do begin
          write(', ');
	  i := i + 1;
          write(v[i])
     end;
     writeln
end;

procedure ClasificacionRapida(VAR v : TDatos; elementos : indice);
	procedure ClasificacionRapidaI(VAR v : TDatos; inf, sup : integer);
    (*
     * Los parametros 'inf' y 'sup' son de tipo 'integer' en lugar de
     * ser de tipo 'indice' porque en las llamadas recursivas que se
     * hacen en esta funcion con los valores ('inf', 'p-1') y
     * ('p+1', 'sup') el valor de 'p+1' podria superar el limite
     * superior del tipo 'indice', mientras que el de 'p-1' podria
     * caer por debajo de su limite inferior, como asi ocurre, de
     * hecho.
     *)
	var
		p : indice;
		procedure Particion(VAR v : TDatos; inf, sup : indice;
                                    VAR pivote : indice);
		var
		   iz, dr : indice;

			procedure Intercambio(VAR v : TDatos; i, j : indice);
			var
			   t : TipoBase;
			begin
			     t := v[i];
			     v[i] := v[j];
			     v[j] := t
			end;
		begin
		     iz  := inf + 1;
		     dr  := sup;
		     while iz <> dr + 1 do begin
	                   while (iz <= dr) and (v[iz] <= v[inf]) do iz := iz + 1;
		           while (iz <= dr) and (v[dr] >= v[inf]) do dr := dr - 1;
		           if iz < dr then begin
		              Intercambio(v, iz, dr);
		              iz := iz + 1;
		              dr := dr - 1
		           end
	             end;
	             Intercambio(v, inf, dr);
		     pivote := dr
		end;

	begin
		if inf < sup then begin
			Particion(v, inf, sup, p);
			ClasificacionRapidaI(v, inf, p-1);
			ClasificacionRapidaI(v, p+1, sup)
		end
	end;

begin
     ClasificacionRapidaI(v, 1, elementos)
end;

begin
     clrscr;
     writeln;
     writeln('*****');
     writeln('***** Algoritmo 7.14_15. Ordenacin rpida de Hoare');
     writeln('*****');
     writeln;
     InitTDatos(t, max_real);
     writeln;
     writeln('Vector antes de la clasificacion:');
     MostrarTDatos(t, max_real);
     write ('Enter para ordenar...');
     readln;
     ClasificacionRapida(t, max_real);
     writeln;
     writeln('Vector despues de la clasificacion:');
     MostrarTDatos(t, max_real);
     Comprueba(t, max_real);
     writeln;
     write ('Pulse enter para continuar');
     readln
end.
