program Algoritmo8_1;
(*
 * Algoritmo 8.1. Comprobar el equilibrio de los signos puntuacion de apertura/cierre
 * Titulo del libro: Una introduccion a la programacion.
 *                   Un enfoque algoritmico
 * Autores del libro: Jesus J. Garcia Molina
 *                    Francisco J. Montoya Dato
 *                    Jose L. Fernandez Aleman
 *                    Maria J. Majado Rosales
 * Fecha: 1/9/2005
 * Capitulo 8. Estructuras de datos dinamicas
 * Ficheros de prueba: dat8_1A.txt, dat8_1B.txt, dat8_1C.txt y dat8_1D.txt
 *
 * SIGNOS
 *
 * Comprueba el equilibrio de los signos de puntuacion parentizados
 * (parentesis, corchetes y llaves) de un fichero cuyo nombre se le
 * solicita por teclado al usuario.
 *
 * A diferencia del algoritmo presentado en el capitulo de EE.DD.
 * dinamicas, este programa no solo dice si el texto es correto o
 * no, sino que en este ultimo caso informa ademas del punto del
 * texto exacto y los signos que causaron el error.
 *
 * Para ello, al recorrer la secuencia de entrada se va llevando
 * cuenta de la posicion por la que en cada momento se va en el reco-
 * rrido (numero de linea y columna dentro de esta). Cuando se en-
 * cuentra un signo de apertura, no solo se guarda en la pila el
 * signo en cuestion, sino tambien el punto (linea y columna) donde
 * este se encontro. De este modo, ante la ocurrencia de un error
 * es posible notificar exactamente los signos que lo produjeron y
 * las posiciones del texto donde estos se encuentran.
 *
 * Para detectar la condicion de error, en lugar de llevar una unica
 * variable booleana llamada 'error', como en el algoritmo del libro,
 * utilizaremos dos variables distintas que nos serviran para identi-
 * ficar el tipo de error exacto. Los nombres y significados de estas
 * variables son los siguientes:
 *
 * error_pila: esta variable se pondra a VERDADERO cuando se encuentre
 *             un signo de cierre y la pila este vacia (es decir, no
 *             haya signos de apertura pendientes de cerrar).
 *
 * error_signo: esta variable se pondra a VERDADERO cuando se encuen-
 *              tre un signo de cierre que no se corresponda con el
 *              ultimo signo de apertura encontrado en el texto.
 *)

uses crt, unitmsc1;

const
     APERTURAS = [ '[', '(', '{' ];  (* conjunto de signos de apertura *)
     CIERRES   = [ ']', ')', '}' ];  (* conjunto de signos de cierre *)

type
    datos_pila =
               record
                     signo : char;
                     linea, columna : integer
               end;

    pila_signos = ^nodo_signos;
    nodo_signos =
                record
                      datos : datos_pila;
                      sig : pila_signos
                end;

var
   S : msc1;
   p : pila_signos;
   d : datos_pila;
   nombre_fichero : string;
   error_signo, error_pila : boolean;
   linea_actual,              (* linea actual del recorrido *)
   columna_actual : integer;  (* columna actual del recorrido *)

(* ---------------- Operaciones de la Pila ----------------- *)

function PilaVacia : pila_signos;
begin
     PilaVacia := NIL
end;

function EsPilaVacia(p : pila_signos) : boolean;
begin
     EsPilaVacia := p = PilaVacia
end;

function Apilar(p : pila_signos; d : datos_pila) : pila_signos;
var
   q : pila_signos;
begin
     new(q);
     q^.datos := d;
     q^.sig   := p;

     Apilar := q
end;

procedure Cima(p : pila_signos; VAR c : datos_pila);
begin
     c := p^.datos
end;

function Desapilar(p : pila_signos) : pila_signos;
var
   q : pila_signos;
begin
     q := p^.sig;
     dispose(p);

     Desapilar := q
end;

function DestruirPila(p : pila_signos) : pila_signos;
begin
     while not EsPilaVacia(p) do p := Desapilar(p);

     DestruirPila := p
end;

(* ---------------- Operaciones del Programa ----------------- *)

function SignoCierre(apertura : char) : char;
(*
 * Devuelve el signo de cierre correspondiente al signo
 * de apertura 'apertura'.
 *
 * Pre: el caracter 'apertura' es un signo de apertura
 *)
var
   res : char;
begin
     case apertura of
          '[' : res := ']';
          '(' : res := ')';
          '{' : res := '}'
     end;
     SignoCierre := res
end;

(* ---------------- Programa Principal ----------------- *)

begin
     clrscr;
     writeln;
     writeln('*****');
     writeln('***** Algoritmo 8.1. Comprobar el equilibrio de los signos puntuacion *****');
     writeln('*****                de apertura/cierre                               *****');
     writeln('*****');
     writeln;
     TratamientoInicial_MSC1(S);
     Cargar_Fichero_MSC1(S, 'dat8_1A.txt');
     writeln('La secuencia de entrada es:');
     writeln;
     Comenzar_MSC1 (S);
     while (EA_MSC1 (S) <> MSC1_MarcaFin) do begin
                if (EA_MSC1(S) = MSC1_MarcaFinLinea) then writeln
                else write(EA_MSC1(S));
       		Avanzar_MSC1 (S)
     end;
     writeln;
     writeln;
     Comenzar_MSC1(S);
     if EA_MSC1(S) = MSC1_MarcaFin then
        writeln('Secuencia vacia.')
     else begin
          p := PilaVacia;
          linea_actual   := 1;
          columna_actual := 1;
          error_signo    := false;
          error_pila     := false;
          repeat
                if EA_MSC1(S) in APERTURAS then begin
                     d.signo   := EA_MSC1(S);
                     d.linea   := linea_actual;
                     d.columna := columna_actual;
                     p := Apilar(p, d)
                end
                else if EA_MSC1(S) in CIERRES then begin
                     if EsPilaVacia(p) then
                        error_pila := true
                     else begin
                          Cima(p, d);
                          p := Desapilar(p);
                          error_signo := EA_MSC1(S) <> SignoCierre(d.signo)
                     end
                end
                else if EA_MSC1(S) = MSC1_MarcaFinLinea then
                begin
                     linea_actual := linea_actual + 1;
                     columna_actual := 0
                end;
                (*
                 * Avanzamos solo si no ha habido error, para preservar
                 * en ese caso los valores de la posicion actual y poder
                 * mostrarlos en el mensaje de error
                 *)
                if not (error_signo or error_pila) then begin
                     Avanzar_MSC1(S);
                     columna_actual := columna_actual + 1
                end
          until (EA_MSC1(S) = MSC1_MarcaFin) or error_signo or error_pila;
          (*
           * Analizamos la razon por la que se detuvo la iteracion:
           *
           * error_pila: se encontro (en el EA) un signo de cierre cuando
           *     no habia pendientes de cierre ningun signo de apertura.
           *
           * error_signo: se encontro (en el EA) un signo de cierre que
           *     no se corresponde con el ultimo signo de apertura
           *     enontrado (que esta en 'd').
           *
           * no hay error, pero la pila no es vacia: el texto se agoto y
           *     los signos de apertura que quedan en la pila no se
           *     cerraron.
           *
           * en otro caso: se alcanzo la marca de fin sin errores, y en
           *     la pila no quedan signos de apertura sin cerrar, luego
           *     el texto es correcto.
           *)
          if error_pila then begin
               write('Error: signo de cierre ''', EA_MSC1(S), ''' en ');
               writeln('linea ', linea_actual, ', columna ', columna_actual, ',');
               writeln('no tiene signo de apertura correspondiente.')
          end
          else if error_signo then begin
               write('Error: signo de cierre ''', EA_MSC1(S), ''' en ');
               writeln('linea ', linea_actual, ', columna ', columna_actual, ',');
               write('no se corresponde con signo de apertura ');
               write('''', d.signo, ''' en linea ', d.linea);
               writeln(', columna ', d.columna);
          end
          else if not EsPilaVacia(p) then begin
               writeln('Error: se agoto el texto con los siguientes ');
               writeln('signos de apertura pendientes de cerrar:');
               writeln;
               repeat
                     Cima(p, d);
                     p := Desapilar(p);
                     write('  signo ''', d.signo, ''': linea ');
                     writeln(d.linea, ', columna ', d.columna)
               until EsPilaVacia(p)
          end
          else
              writeln('Texto correcto.');

          p := DestruirPila(p)
     end;
     writeln;
     write('Pulse enter para continuar');
     readln
end.
