unit unitmsc1;

interface

const
     MSC1_ElemsBuffer   = 200;
     MSC1_MarcaFin      = chr(0);
     MSC1_Tabulador     = chr(9);
     MSC1_MarcaFinLinea = chr(10);
     MSC1_MaxErrores    = 10;

     MSC1_TodoOK            =  0;
     MSC1_CaracterIlegal    =  1;
     MSC1_RegistroIlegal    =  2;
     MSC1_MarcaIlegal       =  3;
     MSC1_ComenzarIlegal    =  4;
     MSC1_AvanzarIlegal     =  5;
     MSC1_EAIlegal          =  6;
     MSC1_NoExFichero       =  7;
     MSC1_ErrCrearFichero   =  8;
     MSC1_ErrLeerFichero    =  9;
     MSC1_ErrEscribFichero  = 10;

type
    MSC1_Errores = 0..MSC1_MaxErrores;
    MSC1_Estados = (MSC1_ninguno, MSC1_creacion, MSC1_marcada, MSC1_consulta);

    MSC1_Indice      = 1..MSC1_ElemsBuffer;
    MSC1_Buffer      = packed array[MSC1_Indice] of char;
    MSC1_PNodoLista  = ^MSC1_NodoLista;
    MSC1_NodoLista   = record
                             buf : MSC1_Buffer;
                             sig : MSC1_PNodoLista;
                       end;

    msc1 = record
                 primero, actual : MSC1_PNodoLista;
                 pos_actual      : integer;
                 elem_actual     : char;
                 estado          : MSC1_Estados;
                 c_error         : MSC1_Errores;
           end;

procedure TratamientoInicial_MSC1(VAR S : msc1);
procedure Arrancar_MSC1(VAR S : msc1);
procedure Registrar_MSC1(VAR S : msc1; c : char);
procedure Marcar_MSC1(VAR S : msc1);
procedure Comenzar_MSC1(VAR S : msc1);
procedure Avanzar_MSC1(VAR S : msc1);
function  EA_MSC1(VAR S : msc1) : char;
procedure Cargar_Fichero_MSC1(VAR S : msc1; nombre : string);
procedure Salvar_Fichero_MSC1(VAR S : msc1; nombre : string);

function  error_MSC1(VAR S : msc1) : MSC1_Errores;
procedure HabilitarErrores_MSC1;
procedure DeshabilitarErrores_MSC1;

implementation

var
   MSC1_ManejarErrores : Boolean;

procedure TratamientoInicial_MSC1(VAR S : msc1);
var
   p : MSC1_PNodoLista;
begin
     new(p);
     p^.sig := NIL;
     with S do
     begin
          primero := p;
          actual := p;
          primero^.buf[1] := MSC1_MarcaFin;
          estado := MSC1_ninguno;
          elem_actual := MSC1_Marcafin;
          pos_actual := 1;
          c_error := MSC1_TodoOK;
     end;
end;

procedure Destruir_MSC1(VAR S : msc1);
var
   p, t : MSC1_PNodoLista;
begin
     p := S.primero^.sig;
     S.primero^.sig := NIL;
     while (p <> NIL) do
     begin
          t := p;
          p := p^.sig;
          dispose(t);
     end;
     S.actual := S.primero;
     S.pos_actual := 1;
     S.primero^.buf[1] := MSC1_MarcaFin;
     S.elem_actual := MSC1_MarcaFin;
     S.estado := MSC1_ninguno;
end;

procedure Arrancar_MSC1(VAR S : msc1);
begin
     Destruir_MSC1(S);
     S.estado := MSC1_creacion;
     S.c_error := MSC1_TodoOK;
end;

procedure CondicionError(VAR S : msc1; err : MSC1_Errores);
begin
     S.c_error := err;
     if MSC1_ManejarErrores then
     begin
          case err of
            MSC1_CaracterIlegal:
                writeln('MSC1: Registrar caracter ilegal');
            MSC1_RegistroIlegal:
                writeln('MSC1: Registrar no en creacion');
            MSC1_MarcaIlegal:
                writeln('MSC1: Marcar ilegal');
            MSC1_ComenzarIlegal:
                writeln('MSC1: Comenzar ilegal');
            MSC1_AvanzarIlegal:
                writeln('MSC1: Avanzar ilegal');
            MSC1_EAIlegal:
                writeln('MSC1: Acceso a EA ilegal');
            MSC1_NoExFichero:
                writeln('MSC1: No existe el fichero');
            MSC1_ErrCrearFichero:
                writeln('MSC1: Imposible crear fichero');
            MSC1_ErrLeerFichero:
                writeln('MSC1: Error al leer del fichero');
            MSC1_ErrEscribFichero:
                writeln('MSC1: Error al escribir en el fichero');
          end;
          halt;
     end;
end;

procedure Registrar_MSC1(VAR S : msc1; c : char);
var
   p : MSC1_PNodoLista;
   permitidos : SET OF char;
begin
     if S.estado = MSC1_creacion then
     begin
          permitidos := [MSC1_MarcaFinLinea, MSC1_Tabulador];
          if (c >= ' ') or (c in permitidos) then
          begin
               S.actual^.buf[S.pos_actual] := c;
               inc(S.pos_actual);
               if S.pos_actual > MSC1_ElemsBuffer then
               begin
                    new(p);
                    S.pos_actual  := 1;
                    S.actual^.sig := p;
                    S.actual      := p;
                    p^.sig        := NIL;
               end;
          end
          else
              CondicionError(S, MSC1_CaracterIlegal);
     end
     else
         CondicionError(S, MSC1_RegistroIlegal);
end;

procedure Marcar_MSC1(VAR S : msc1);
begin
     if S.estado = MSC1_creacion then
     begin
          S.actual^.buf[S.pos_actual] := MSC1_MarcaFin;
          S.estado := MSC1_marcada;
     end
     else
         CondicionError(S, MSC1_MarcaIlegal);
end;

procedure Comenzar_MSC1(VAR S : msc1);
begin
     if S.estado in [MSC1_consulta,MSC1_ninguno,MSC1_marcada] then
     begin
          S.actual := S.primero;
          S.pos_actual := 1;
          S.elem_actual := S.actual^.buf[1];
          S.estado := MSC1_consulta;
     end
     else
         CondicionError(S, MSC1_ComenzarIlegal);
end;

procedure Avanzar_MSC1(VAR S : msc1);
begin
     if (S.estado = MSC1_consulta) and (S.elem_actual <> MSC1_MarcaFin) then
     begin
          inc(S.pos_actual);
          if S.pos_actual > MSC1_ElemsBuffer then
          begin
               S.pos_actual := 1;
               S.actual := S.actual^.sig;
          end;
          S.elem_actual := S.actual^.buf[S.pos_actual];
     end
     else
         CondicionError(S, MSC1_AvanzarIlegal);
end;

function EA_MSC1(VAR S : msc1) : char;
begin
     if S.estado = MSC1_consulta
     then
         EA_MSC1 := S.elem_actual
     else
     begin
          CondicionError(S, MSC1_EAIlegal);
          EA_MSC1 := MSC1_MarcaFin;
     end;
end;

function error_MSC1(var S : msc1) : MSC1_Errores;
var
   r : MSC1_Errores;
begin
     r := S.c_error;
     S.c_error := MSC1_TodoOK;
     error_MSC1 := r;
end;

procedure Cargar_Fichero_MSC1(var S : msc1; nombre : string);
var
   f : text;
   c : char;
   read_error : boolean;
begin
     assign(f, nombre);
     {$I-}
     reset(f);
     {$I+}
     if IOResult <> 0 then
          CondicionError(S, MSC1_NoExFichero)
     else
     begin
          S.c_error := MSC1_TodoOk;
          Arrancar_MSC1(S);
          read_error := false;
          while not eof(f) and not read_error and
                (S.c_error = MSC1_TodoOk) do
          begin
               while not eoln(f) do
               begin
                    {$I-}
                    read(f, c);
                    {$I+}
                    if IOResult <> 0 then
                    begin
                         read_error := true;
                         CondicionError(S, MSC1_ErrLeerFichero);
                    end
                    else
                        Registrar_MSC1(S, c);
               end;
               if not eof(f) then Registrar_MSC1(S, MSC1_MarcaFinLinea);
               {$I-}
               readln(f);
               {$I+}
               if IOResult <> 0 then
               begin
                    read_error := true;
                    CondicionError(S, MSC1_ErrLeerFichero);
               end;
          end;
          Marcar_MSC1(S);
          close(f);
     end;
end;

procedure Salvar_Fichero_MSC1(VAR S : msc1; nombre : string);
var
   f : text;
   write_error : boolean;
begin
     assign(f, nombre);
     {$I-}
     rewrite(f);
     {$I+}
     if IOResult <> 0 then
        CondicionError(S, MSC1_ErrCrearFichero)
     else
     begin
          S.c_error := MSC1_TodoOk;
          write_error := false;
          Comenzar_MSC1(S);
          while (EA_MSC1(S) <> MSC1_MarcaFin) and not write_error do
          begin
               while (EA_MSC1(S) <> MSC1_MarcaFinLinea) and
                     not write_error do
               begin
                    {$I-}
                    write(f, EA_MSC1(S));
                    {$I+}
                    if IOResult <> 0 then
                    begin
                         write_error := true;
                         CondicionError(S, MSC1_ErrEscribFichero);
                    end
                    else
                        Avanzar_MSC1(S);
               end;
               {$I-}
               writeln(f);
               {$I+}
               if IOResult <> 0 then
               begin
                    write_error := true;
                    CondicionError(S, MSC1_ErrEscribFichero);
               end
               else
                   Avanzar_MSC1(S);
          end;
          close(f);
     end;
end;

procedure HabilitarErrores_MSC1;
begin
     MSC1_ManejarErrores := true;
end;

procedure DeshabilitarErrores_MSC1;
begin
     MSC1_ManejarErrores := false;
end;

(* Rutinas de inicializacion *)

begin
     MSC1_ManejarErrores := true;
end.