unit unitmsc2;

interface

const
     MSC2_ElemsBuffer   = 200;
     MSC2_MarcaFinLinea = chr(10);
     MSC2_Tabulador     = chr(9);
     MSC2_MaxErrores    = 9;

     MSC2_TodoOK          = 0;
     MSC2_CaracterIlegal  = 1;
     MSC2_RegistroIlegal  = 2;
     MSC2_ComenzarIlegal  = 3;
     MSC2_AvanzarIlegal   = 4;
     MSC2_EAIlegal        = 5;
     MSC2_NoExFichero     = 6;
     MSC2_ErrCrearFichero = 7;
     MSC2_ErrLeerFichero  = 8;
     MSC2_ErrEscribFichero= 9;

type
    MSC2_Errores = 0..MSC2_MaxErrores;
    MSC2_Estados = (MSC2_ninguno, MSC2_iniacc, MSC2_creacion, MSC2_consulta);

    MSC2_Indice      = 1..MSC2_ElemsBuffer;
    MSC2_Buffer      = packed array[MSC2_Indice] of char;
    MSC2_PNodoLista  = ^MSC2_NodoLista;
    MSC2_NodoLista   = record
                             buf  : MSC2_Buffer;
                             ocup : integer;
                             sig  : MSC2_PNodoLista;
                       end;

    msc2 = record
                 primero, actual : MSC2_PNodoLista;
                 pos_actual      : integer;
                 elem_actual     : char;
                 estado          : MSC2_Estados;
                 c_error         : MSC2_Errores;
           end;

procedure  TratamientoInicial_MSC2(VAR S : msc2);
function   EsVacia_MSC2(S : msc2) : boolean;
procedure  Arrancar_MSC2(VAR S : msc2);
procedure  Registrar_MSC2(VAR S : msc2; c : char);
function   EsUltimo_MSC2(S : msc2) : boolean;
procedure  InicAcceso_MSC2(VAR S : msc2);
procedure  Avanzar_MSC2(VAR S : msc2);
function   EA_MSC2(VAR S : msc2) : char;
procedure  Cargar_Fichero_MSC2(VAR S : msc2; nombre : string);
procedure  Salvar_Fichero_MSC2(VAR S : msc2; nombre : string);

function   error_MSC2(VAR S : msc2) : MSC2_Errores;
procedure  HabilitarErrores_MSC2;
procedure  DeshabilitarErrores_MSC2;

implementation

var
   MSC2_ManejarErrores : boolean;

procedure TratamientoInicial_MSC2(VAR S : msc2);
begin
     with S do
     begin
          primero    := NIL;
          actual     := NIL;
          estado     := MSC2_ninguno;
          c_error    := MSC2_TodoOK;
          pos_actual := 1;
     end;
end;

procedure CondicionError(VAR S : msc2; err : MSC2_Errores);
begin
     S.c_error := err;
     if MSC2_ManejarErrores then
     begin
          case err of
            MSC2_CaracterIlegal:
                writeln('MSC2: Registrar caracter ilegal');
            MSC2_RegistroIlegal:
                writeln('MSC2: Registrar no en creacion');
            MSC2_ComenzarIlegal:
                writeln('MSC2: Comenzar ilegal');
            MSC2_AvanzarIlegal:
                writeln('MSC2: Avanzar ilegal');
            MSC2_EAIlegal:
                writeln('MSC2: Acceso a EA ilegal');
            MSC2_NoExFichero:
                writeln('MSC2: No existe el fichero');
            MSC2_ErrCrearFichero:
                writeln('MSC2: Imposible crear fichero');
            MSC2_ErrLeerFichero:
                writeln('MSC2: Error al leer del fichero');
            MSC2_ErrEscribFichero:
                writeln('MSC2: Error al escribir en el fichero');
          end;
          halt;
     end;
end;


function EsVacia_MSC2(S : msc2) : boolean;
begin
     EsVacia_MSC2 := S.primero = NIL;
end;

procedure Destruir_MSC2(VAR S : msc2);
var
   p, t : MSC2_PNodoLista;
begin
     with S do
     begin
          p       := primero;
          primero := NIL;
          actual  := NIL;
          while p <> NIL do
          begin
               t := p;
               p := p^.sig;
               dispose(t);
          end;
          estado     := MSC2_ninguno;
          c_error    := MSC2_TodoOK;
          pos_actual := 1;
     end;
end;

procedure Arrancar_MSC2(VAR S : msc2);
begin
     Destruir_MSC2(S);
     S.estado := MSC2_creacion;
end;

procedure Registrar_MSC2(VAR S : msc2; c : char);
var
   p : MSC2_PNodoLista;
   permitidos : SET OF char;
begin
     if S.estado = MSC2_creacion then
     begin
        permitidos := [MSC2_MarcaFinLinea, MSC2_Tabulador];
        if (c >= ' ') or (c in permitidos) then
        with S do
        begin
          if primero = NIL then
          begin
               new(p);
               primero    := p;
               actual     := p;
               p^.ocup    := 0;
               p^.sig     := NIL;
               pos_actual := 0;
          end;
          if actual^.ocup = MSC2_ElemsBuffer then
          begin
               new(p);
               actual^.sig := p;
               actual      := p;
               p^.ocup     := 0;
               p^.sig      := NIL;
               pos_actual  := 0;
          end;
          inc(pos_actual);
          inc(actual^.ocup);
          actual^.buf[pos_actual] := c;
        end
        else
          CondicionError(S, MSC2_CaracterIlegal)
     end
     else
         CondicionError(S, MSC2_RegistroIlegal);
end;

function EsUltimo_MSC2(S : msc2) : boolean;
begin
     if EsVacia_MSC2(S) then
        EsUltimo_MSC2 := true
     else
     with S do
       EsUltimo_MSC2 := (pos_actual = actual^.ocup) and (actual^.sig = NIL);
end;

procedure InicAcceso_MSC2(VAR S : msc2);
begin
     with S do
     begin
          actual := primero;
          pos_actual := 0;
          estado := MSC2_iniacc;
     end;
end;

procedure Avanzar_MSC2(VAR S : msc2);
begin
     if (S.estado in [MSC2_consulta,MSC2_iniacc]) and not EsUltimo_MSC2(S)
     then
       with S do
           begin
                estado := MSC2_consulta;
                inc(pos_actual);
                if (pos_actual > MSC2_ElemsBuffer) then
                begin
                     pos_actual := 1;
                     actual := actual^.sig;
                end;
                elem_actual := actual^.buf[pos_actual];
           end
        else
           CondicionError(S, MSC2_AvanzarIlegal);
end;

function EA_MSC2(VAR S : msc2) : char;
begin
     if S.estado = MSC2_consulta then
         EA_MSC2 := S.elem_actual
     else
     begin
          CondicionError(S, MSC2_EAIlegal);
          EA_MSC2 := '?';
     end;
end;

function error_MSC2(var S : msc2) : MSC2_Errores;
var
   err : MSC2_Errores;
begin
     err := S.c_error;
     S.c_error := MSC2_TodoOK;
     error_MSC2 := err;
end;

procedure Cargar_Fichero_MSC2(VAR S : msc2; nombre : string);
var
   f : text;
   c : char;
   read_error : boolean;
begin
     assign(f, nombre);
     {$I-}
     reset(f);
     {$I+}
     if IOResult <> 0 then
        CondicionError(S, MSC2_NoExFichero)
     else
     begin
          S.c_error := MSC2_TodoOk;
          read_error := false;
          Arrancar_MSC2(S);
          while not eof(f) and not read_error and
                (S.c_error = MSC2_TodoOk) do
          begin
               while not eoln(f) and not read_error and
                     (S.c_error = MSC2_TodoOk) do
               begin
                    {$I-}
                    read(f, c);
                    {$I+}
                    if IOResult <> 0 then
                    begin
                         read_error := true;
                         CondicionError(S, MSC2_ErrLeerFichero);
                    end
                    else
                        Registrar_MSC2(S, c);
               end;
               if not eof(f) then Registrar_MSC2(S, MSC2_MarcaFinLinea);
               {$I-}
               readln(f);
               {$I+}
               if IOResult <> 0 then
               begin
                    read_error := true;
                    CondicionError(S, MSC2_ErrLeerFichero);
               end;
          end;
          close(f);
     end;
end;

procedure Salvar_Fichero_MSC2(VAR S : msc2; nombre : string);
var
   f : text;
   write_error : boolean;
begin
     assign(f, nombre);
     {$I-}
     rewrite(f);
     {$I+}
     if IOResult <> 0 then
        CondicionError(S, MSC2_ErrCrearFichero)
     else
     begin
          S.c_error := MSC2_TodoOk;
          write_error := false;
          InicAcceso_MSC2(S);
          case EsVacia_MSC2(S) of
               true  :  ;
               false : repeat
                             Avanzar_MSC2(S);
                             {$I-}
                             if EA_MSC2(S) = MSC2_MarcaFinLinea then
                                writeln(f)
                             else
                                 write(f, EA_MSC2(S));
                             {$I+}
                             if IOResult <> 0 then
                             begin
                                  write_error := true;
                                  CondicionError(S, MSC2_ErrEscribFichero);
                             end;
                       until EsUltimo_MSC2(S) or write_error;
          end;
          close(f);
     end;
end;

procedure HabilitarErrores_MSC2;
begin
     MSC2_ManejarErrores := true;
end;

procedure DeshabilitarErrores_MSC2;
begin
     MSC2_ManejarErrores := false;
end;


(* Rutinas de inicializacion *)
begin
     MSC2_ManejarErrores := true;
end.