unit unitmse1;

interface

const
     MSE1_ElemsBuffer   =    200;
     MSE1_MarcaFin      = -10000;
     MSE1_MaxErrores    =     10;
     MSE1_MinValueAllow =  -1000;
     MSE1_MaxValueAllow =   1000;

     MSE1_TodoOK             =  0;
     MSE1_EnteroIlegal       =  1;
     MSE1_RegistroIlegal     =  2;
     MSE1_MarcaIlegal        =  3;
     MSE1_ComenzarIlegal     =  4;
     MSE1_AvanzarIlegal      =  5;
     MSE1_EAIlegal           =  6;
     MSE1_NoExFichero        =  7;
     MSE1_ErrCrearFichero    =  8;
     MSE1_ErrLeerFichero     =  9;
     MSE1_ErrEscribirFichero = 10;

type
    MSE1_Errores = 0..MSE1_MaxErrores;
    MSE1_Estados = (MSE1_ninguno, MSE1_creacion, MSE1_marcada, MSE1_consulta);

    MSE1_Indice      = 1..MSE1_ElemsBuffer;
    MSE1_Buffer      = packed array[MSE1_Indice] of integer;
    MSE1_PNodoLista  = ^MSE1_NodoLista;
    MSE1_NodoLista   = record
                             buf : MSE1_Buffer;
                             sig : MSE1_PNodoLista;
                       end;

    mse1 = record
                 primero, actual : MSE1_PNodoLista;
                 pos_actual      : integer;
                 elem_actual     : integer;
                 estado          : MSE1_Estados;
                 c_error         : MSE1_Errores;
           end;

procedure TratamientoInicial_MSE1(VAR S : mse1);
procedure Arrancar_MSE1(VAR S : mse1);
procedure Registrar_MSE1(VAR S : mse1; c : integer);
procedure Marcar_MSE1(VAR S : mse1);
procedure Comenzar_MSE1(VAR S : mse1);
procedure Avanzar_MSE1(VAR S : mse1);
function  EA_MSE1(VAR S : mse1) : integer;
procedure Cargar_Fichero_MSE1(VAR S : mse1; nombre : string);
procedure Salvar_Fichero_MSE1(VAR S : mse1; nombre : string);

function  error_MSE1(VAR S : mse1) : MSE1_Errores;
procedure HabilitarErrores_MSE1;
procedure DeshabilitarErrores_MSE1;

implementation

var
   MSE1_ManejarErrores : Boolean;

procedure TratamientoInicial_MSE1(VAR S : mse1);
var
   p : MSE1_PNodoLista;
begin
     new(p);
     p^.sig := NIL;
     with S do
     begin
          primero := p;
          actual := p;
          primero^.buf[1] := MSE1_MarcaFin;
          estado := MSE1_ninguno;
          elem_actual := MSE1_MarcaFin;
          pos_actual := 1;
          c_error := MSE1_TodoOK;
     end;
end;

procedure Destruir_MSE1(VAR S : mse1);
var
   p, t : MSE1_PNodoLista;
begin
     p := S.primero^.sig;
     S.primero^.sig := NIL;
     while (p <> NIL) do
     begin
          t := p;
          p := p^.sig;
          dispose(t);
     end;
     S.actual := S.primero;
     S.pos_actual := 1;
     S.primero^.buf[1] := MSE1_MarcaFin;
     S.elem_actual := MSE1_MarcaFin;
     S.estado := MSE1_ninguno;
end;

procedure Arrancar_MSE1(VAR S : mse1);
begin
     Destruir_MSE1(S);
     S.estado := MSE1_creacion;
     S.c_error := MSE1_TodoOK;
end;

procedure CondicionError(VAR S : mse1; err : MSE1_Errores);
begin
     S.c_error := err;
     if MSE1_ManejarErrores then
     begin
          case err of
            MSE1_EnteroIlegal:
                writeln('MSE1: Registrar Entero ilegal');
            MSE1_RegistroIlegal:
                writeln('MSE1: Registrar no en creacion');
            MSE1_MarcaIlegal:
                writeln('MSE1: Marcar ilegal');
            MSE1_ComenzarIlegal:
                writeln('MSE1: Comenzar ilegal');
            MSE1_AvanzarIlegal:
                writeln('MSE1: Avanzar ilegal');
            MSE1_EAIlegal:
                writeln('MSE1: Acceso a EA ilegal');
            MSE1_NoExFichero:
                writeln('MSE1: No existe el fichero');
            MSE1_ErrCrearFichero:
                writeln('MSE1: Imposible crear fichero');
            MSE1_ErrLeerFichero:
                writeln('MSE1: Error al leer datos del fichero');
            MSE1_ErrEscribirFichero:
                writeln('MSE1: Error al escribir datos en el fichero');
          end;
          halt;
     end;
end;

procedure Registrar_MSE1(VAR S : mse1; c : integer);
var
   p : MSE1_PNodoLista;
begin
     if S.estado = MSE1_creacion then
     begin
          if (c >= MSE1_MinValueAllow) and (c <= MSE1_MaxValueAllow)  then
          begin
               S.actual^.buf[S.pos_actual] := c;
               inc(S.pos_actual);
               if S.pos_actual > MSE1_ElemsBuffer then
               begin
                    new(p);
                    S.pos_actual  := 1;
                    S.actual^.sig := p;
                    S.actual      := p;
                    p^.sig        := NIL;
               end;
          end
          else
              CondicionError(S, MSE1_EnteroIlegal);
     end
     else
         CondicionError(S, MSE1_RegistroIlegal);
end;

procedure Marcar_MSE1(VAR S : mse1);
begin
     if S.estado = MSE1_creacion then
     begin
          S.actual^.buf[S.pos_actual] := MSE1_MarcaFin;
          S.estado := MSE1_marcada;
     end
     else
         CondicionError(S, MSE1_MarcaIlegal);
end;

procedure Comenzar_MSE1(VAR S : mse1);
begin
     if S.estado in [MSE1_consulta,MSE1_ninguno,MSE1_marcada] then
     begin
          S.actual := S.primero;
          S.pos_actual := 1;
          S.elem_actual := S.actual^.buf[1];
          S.estado := MSE1_consulta;
     end
     else
         CondicionError(S, MSE1_ComenzarIlegal);
end;

procedure Avanzar_MSE1(VAR S : mse1);
begin
     if (S.estado = MSE1_consulta) and (S.elem_actual <> MSE1_MarcaFin) then
     begin
          inc(S.pos_actual);
          if S.pos_actual > MSE1_ElemsBuffer then
          begin
               S.pos_actual := 1;
               S.actual := S.actual^.sig;
          end;
          S.elem_actual := S.actual^.buf[S.pos_actual];
     end
     else
         CondicionError(S, MSE1_AvanzarIlegal);
end;

function EA_MSE1(VAR S : mse1) : integer;
begin
     if S.estado = MSE1_consulta
     then
         EA_MSE1 := S.elem_actual
     else
     begin
          CondicionError(S, MSE1_EAIlegal);
          EA_MSE1 := MSE1_MarcaFin;
     end;
end;

function error_MSE1(var S : mse1) : MSE1_Errores;
var
   r : MSE1_Errores;
begin
     r := S.c_error;
     S.c_error := MSE1_TodoOK;
     error_MSE1 := r;
end;

procedure Cargar_Fichero_MSE1(var S : mse1; nombre : string);
var
   f : text;
   c : integer;
   input_error : boolean;

   procedure salta_lineas_blancas(var Se : mse1;
                                  var fi : text;
                                  var r_error : boolean);
   begin
        while eoln(fi) and not eof(fi) and not r_error
              and (Se.c_error = MSE1_TodoOk) do
        begin
             {$I-}
             readln(fi);
             {$I+}
             if IOResult <> 0 then
             begin
                  r_error := true;
                  CondicionError(Se, MSE1_ErrLeerFichero);
             end;
        end;
   end;

begin
     S.c_error := MSE1_TodoOk;
     assign(f, nombre);
     {$I-}
     reset(f);
     {$I+}
     if IOResult <> 0 then
          CondicionError(S, MSE1_NoExFichero)
     else
     begin
          Arrancar_MSE1(S);
          input_error := false;
          while not eof(f) and not input_error and
                (S.c_error = MSE1_TodoOk) do
          begin
               {$I-}
               readln(f, c);
               {$I+}
               if IOResult <> 0 then
               begin
                    input_error := true;
                    CondicionError(S, MSE1_ErrLeerFichero);
               end
               else
                   Registrar_MSE1(S, c);
               salta_lineas_blancas(S, f, input_error);
          end;
          Marcar_MSE1(S);
          close(f);
     end;
end;

procedure Salvar_Fichero_MSE1(VAR S : mse1; nombre : string);
var
   f : text;
   write_error : boolean;
begin
     S.c_error := MSE1_TodoOk;
     assign(f, nombre);
     {$I-}
     rewrite(f);
     {$I+}
     if IOResult <> 0 then
        CondicionError(S, MSE1_ErrCrearFichero)
     else
     begin
          Comenzar_MSE1(S);
          write_error := false;
          while (EA_MSE1(S) <> MSE1_MarcaFin) and
                not write_error and (S.c_error = MSE1_TodoOk) do
          begin
               {$I-}
               writeln(f, EA_MSE1(S));
               {$I+}
               if IOResult <> 0 then
               begin
                    write_error := true;
                    CondicionError(S, MSE1_ErrEscribirFichero);
               end
               else
                   Avanzar_MSE1(S);
          end;
          close(f);
     end;
end;

procedure HabilitarErrores_MSE1;
begin
     MSE1_ManejarErrores := true;
end;

procedure DeshabilitarErrores_MSE1;
begin
     MSE1_ManejarErrores := false;
end;

(* Rutinas de inicializacion *)

begin
     MSE1_ManejarErrores := true;
end.
