unit unitmse2;

interface

const
     MSE2_ElemsBuffer   = 200;
     MSE2_MaxErrores    = 9;

     MSE2_TodoOK           = 0;
     MSE2_EnteroIlegal     = 1;
     MSE2_RegistroIlegal   = 2;
     MSE2_ComenzarIlegal   = 3;
     MSE2_AvanzarIlegal    = 4;
     MSE2_EAIlegal         = 5;
     MSE2_NoExistFichero   = 6;
     MSE2_ErrCrearFichero  = 7;
     MSE2_ErrLeerFichero   = 8;
     MSE2_ErrEscribFichero = 9;

     MSE2_MaxValueAllow  =  1000;
     MSE2_MinValueAllow  = -1000;

type
    MSE2_Errores = 0..MSE2_MaxErrores;
    MSE2_Estados = (MSE2_ninguno, MSE2_iniacc, MSE2_creacion, MSE2_consulta);

    MSE2_Indice      = 1..MSE2_ElemsBuffer;
    MSE2_Buffer      = packed array[MSE2_Indice] of integer;
    MSE2_PNodoLista  = ^MSE2_NodoLista;
    MSE2_NodoLista   = record
                             buf  : MSE2_Buffer;
                             ocup : integer;
                             sig  : MSE2_PNodoLista;
                       end;

    mse2 = record
                 primero, actual : MSE2_PNodoLista;
                 pos_actual      : integer;
                 elem_actual     : integer;
                 estado          : MSE2_Estados;
                 c_error         : MSE2_Errores;
           end;

procedure  TratamientoInicial_MSE2(VAR S : mse2);
function   EsVacia_MSE2(S : mse2) : boolean;
procedure  Arrancar_MSE2(VAR S : mse2);
procedure  Registrar_MSE2(VAR S : mse2; c : integer);
function   EsUltimo_MSE2(S : mse2) : boolean;
procedure  InicAcceso_MSE2(VAR S : mse2);
procedure  Avanzar_MSE2(VAR S : mse2);
function   EA_MSE2(VAR S : mse2) : integer;
procedure  Cargar_Fichero_MSE2(VAR S : mse2; nombre : string);
procedure  Salvar_Fichero_MSE2(VAR S : mse2; nombre : string);

function   error_MSE2(VAR S : mse2) : MSE2_Errores;
procedure  HabilitarErrores_MSE2;
procedure  DeshabilitarErrores_MSE2;

implementation

var
   MSE2_ManejarErrores : boolean;

procedure TratamientoInicial_MSE2(VAR S : mse2);
begin
     with S do
     begin
          primero    := NIL;
          actual     := NIL;
          estado     := MSE2_ninguno;
          c_error    := MSE2_TodoOK;
          pos_actual := 1;
     end;
end;

procedure CondicionError(VAR S : mse2; err : MSE2_Errores);
begin
     S.c_error := err;
     if MSE2_ManejarErrores then
     begin
          case err of
            MSE2_EnteroIlegal:
                writeln('MSE2: Registrar entero ilegal');
            MSE2_RegistroIlegal:
                writeln('MSE2: Registrar no en creacion');
            MSE2_ComenzarIlegal:
                writeln('MSE2: Comenzar ilegal');
            MSE2_AvanzarIlegal:
                writeln('MSE2: Avanzar ilegal');
            MSE2_EAIlegal:
                writeln('MSE2: Acceso a EA ilegal');
            MSE2_NoExistFichero:
                writeln('MSE2: No existe el fichero');
            MSE2_ErrCrearFichero:
                writeln('MSE2: Imposible crear fichero');
            MSE2_ErrLeerFichero:
                writeln('MSE2: Error leyendo del fichero');
            MSE2_ErrEscribFichero:
                writeln('MSE2: Error escribiendo en el fichero');
          end;
          halt;
     end;
end;


function EsVacia_MSE2(S : mse2) : boolean;
begin
     EsVacia_MSE2 := S.primero = NIL;
end;

procedure Destruir_MSE2(VAR S : mse2);
var
   p, t : MSE2_PNodoLista;
begin
     with S do
     begin
          p       := primero;
          primero := NIL;
          actual  := NIL;
          while p <> NIL do
          begin
               t := p;
               p := p^.sig;
               dispose(t);
          end;
          estado     := MSE2_ninguno;
          c_error    := MSE2_TodoOK;
          pos_actual := 1;
     end;
end;

procedure Arrancar_MSE2(VAR S : mse2);
begin
     Destruir_MSE2(S);
     S.estado := MSE2_creacion;
end;

procedure Registrar_MSE2(VAR S : mse2; c : integer);
var
   p : MSE2_PNodoLista;
begin
     if S.estado = MSE2_creacion then
     begin
        if (c >= MSE2_MinValueAllow) and
           (c <= MSE2_MaxValueAllow) then
        with S do
        begin
          if primero = NIL then
          begin
               new(p);
               primero    := p;
               actual     := p;
               p^.ocup    := 0;
               p^.sig     := NIL;
               pos_actual := 0;
          end;
          if actual^.ocup = MSE2_ElemsBuffer then
          begin
               new(p);
               actual^.sig := p;
               actual      := p;
               p^.ocup     := 0;
               p^.sig      := NIL;
               pos_actual  := 0;
          end;
          inc(pos_actual);
          inc(actual^.ocup);
          actual^.buf[pos_actual] := c;
        end
        else
          CondicionError(S, MSE2_EnteroIlegal)
     end
     else
         CondicionError(S, MSE2_RegistroIlegal);
end;

function EsUltimo_MSE2(S : mse2) : boolean;
begin
     if EsVacia_MSE2(S) then
        EsUltimo_MSE2 := true
     else
     with S do
       EsUltimo_MSE2 := (pos_actual = actual^.ocup) and (actual^.sig = NIL);
end;

procedure InicAcceso_MSE2(VAR S : mse2);
begin
     with S do
     begin
          actual := primero;
          pos_actual := 0;
          estado := MSE2_iniacc;
     end;
end;

procedure Avanzar_MSE2(VAR S : mse2);
begin
     if (S.estado in [MSE2_consulta,MSE2_iniacc]) and not EsUltimo_MSE2(S)
     then
       with S do
           begin
                estado := MSE2_consulta;
                inc(pos_actual);
                if (pos_actual > MSE2_ElemsBuffer) then
                begin
                     pos_actual := 1;
                     actual := actual^.sig;
                end;
                elem_actual := actual^.buf[pos_actual];
           end
        else
           CondicionError(S, MSE2_AvanzarIlegal);
end;

function EA_MSE2(VAR S : mse2) : integer;
begin
     if S.estado = MSE2_consulta then
         EA_MSE2 := S.elem_actual
     else
     begin
          CondicionError(S, MSE2_EAIlegal);
          EA_MSE2 := 0;
     end;
end;

procedure  Cargar_Fichero_MSE2(VAR S : mse2; nombre : string);
var
   f : text;
   read_error : boolean;
   c : integer;

   procedure salta_lineas_blancas(VAR Se : mse2;
                                  VAR fi : text;
                                  VAR err : boolean);
   begin
        while eoln(fi) and not eof(fi) and not err do
        begin
             {$I-}
             readln(fi);
             {$I+}
             if IOResult <> 0 then
             begin
                  err := true;
                  Se.c_error := MSE2_ErrLeerFichero;
             end;
        end;
   end;

begin
     assign(f, nombre);
     {$I-}
     reset(f);
     {$I+}
     if IOResult <> 0 then
        CondicionError(S, MSE2_NoExistFichero)
     else
         begin
              S.c_error := MSE2_TodoOk;
              read_error := false;
              Arrancar_MSE2(S);
              while not eof(f) and not read_error and
                    (S.c_error = MSE2_TodoOk) do
              begin
                   {$I-}
                   readln(f, c);
                   {$I+}
                   if IOResult <> 0 then
                   begin
                        read_error := true;
                        CondicionError(S, MSE2_ErrLeerFichero);
                   end
                   else
                   begin
                        Registrar_MSE2(S, c);
                        salta_lineas_blancas(S, f, read_error);
                   end;
              end;
              close(f);
         end;
end;

procedure  Salvar_Fichero_MSE2(VAR S : mse2; nombre : string);
var
   f : text;
   write_error : boolean;
begin
     assign(f, nombre);
     {$I-}
     rewrite(f);
     {$I+}
     if IOResult <> 0 then
        CondicionError(S, MSE2_ErrCrearFichero)
     else
         begin
              S.c_error := MSE2_TodoOk;
              write_error := false;
              InicAcceso_MSE2(S);
              while not write_error and
                    (S.c_error = MSE2_TodoOk) and
                    not EsUltimo_MSE2(S) do
              begin
                   Avanzar_MSE2(S);
                   {$I-}
                   writeln(f, EA_MSE2(S));
                   {$I+}
                   if IOResult <> 0 then
                   begin
                        write_error := true;
                        CondicionError(S, MSE2_ErrEscribFichero);
                   end;
              end;
              close(f);
         end;
end;

function error_MSE2(VAR S : mse2) : MSE2_Errores;
var
   err : MSE2_Errores;
begin
     err := S.c_error;
     S.c_error := MSE2_TodoOK;
     error_MSE2 := err;
end;

procedure HabilitarErrores_MSE2;
begin
     MSE2_ManejarErrores := true;
end;

procedure DeshabilitarErrores_MSE2;
begin
     MSE2_ManejarErrores := false;
end;


(* Rutinas de inicializacion *)
begin
     MSE2_ManejarErrores := true;
end.
