unit unitmsr1;

interface

const
     MSR1_ElemsBuffer   =    200;
     MSR1_MarcaFin      = -10000;
     MSR1_MaxErrores    =     10;
     MSR1_MinValueAllow =  -1000;
     MSR1_MaxValueAllow =   1000;

     MSR1_TodoOK             =  0;
     MSR1_RealIlegal       =  1;
     MSR1_RegistroIlegal     =  2;
     MSR1_MarcaIlegal        =  3;
     MSR1_ComenzarIlegal     =  4;
     MSR1_AvanzarIlegal      =  5;
     MSR1_EAIlegal           =  6;
     MSR1_NoExFichero        =  7;
     MSR1_ErrCrearFichero    =  8;
     MSR1_ErrLeerFichero     =  9;
     MSR1_ErrEscribirFichero = 10;

type
    MSR1_Errores = 0..MSR1_MaxErrores;
    MSR1_Estados = (MSR1_ninguno, MSR1_creacion, MSR1_marcada, MSR1_consulta);

    MSR1_Indice      = 1..MSR1_ElemsBuffer;
    MSR1_Buffer      = packed array[MSR1_Indice] of real;
    MSR1_PNodoLista  = ^MSR1_NodoLista;
    MSR1_NodoLista   = record
                             buf : MSR1_Buffer;
                             sig : MSR1_PNodoLista;
                       end;

    msr1 = record
                 primero, actual : MSR1_PNodoLista;
                 pos_actual      : integer;
                 elem_actual     : real;
                 estado          : MSR1_Estados;
                 c_error         : MSR1_Errores;
           end;

procedure TratamientoInicial_MSR1(VAR S : msr1);
procedure Arrancar_MSR1(VAR S : msr1);
procedure Registrar_MSR1(VAR S : msr1; c : real);
procedure Marcar_MSR1(VAR S : msr1);
procedure Comenzar_MSR1(VAR S : msr1);
procedure Avanzar_MSR1(VAR S : msr1);
function  EA_MSR1(VAR S : msr1) : real;
procedure Cargar_Fichero_MSR1(VAR S : msr1; nombre : string);
procedure Salvar_Fichero_MSR1(VAR S : msr1; nombre : string);

function  error_MSR1(VAR S : msr1) : MSR1_Errores;
procedure HabilitarErrores_MSR1;
procedure DeshabilitarErrores_MSR1;

implementation

var
   MSR1_ManejarErrores : Boolean;

procedure TratamientoInicial_MSR1(VAR S : msr1);
var
   p : MSR1_PNodoLista;
begin
     new(p);
     p^.sig := NIL;
     with S do
     begin
          primero := p;
          actual := p;
          primero^.buf[1] := MSR1_MarcaFin;
          estado := MSR1_ninguno;
          elem_actual := MSR1_MarcaFin;
          pos_actual := 1;
          c_error := MSR1_TodoOK;
     end;
end;

procedure Destruir_MSR1(VAR S : msr1);
var
   p, t : MSR1_PNodoLista;
begin
     p := S.primero^.sig;
     S.primero^.sig := NIL;
     while (p <> NIL) do
     begin
          t := p;
          p := p^.sig;
          dispose(t);
     end;
     S.actual := S.primero;
     S.pos_actual := 1;
     S.primero^.buf[1] := MSR1_MarcaFin;
     S.elem_actual := MSR1_MarcaFin;
     S.estado := MSR1_ninguno;
end;

procedure Arrancar_MSR1(VAR S : msr1);
begin
     Destruir_MSR1(S);
     S.estado := MSR1_creacion;
     S.c_error := MSR1_TodoOK;
end;

procedure CondicionError(VAR S : msr1; err : MSR1_Errores);
begin
     S.c_error := err;
     if MSR1_ManejarErrores then
     begin
          case err of
            MSR1_RealIlegal:
                writeln('MSR1: Registrar Entero ilegal');
            MSR1_RegistroIlegal:
                writeln('MSR1: Registrar no en creacion');
            MSR1_MarcaIlegal:
                writeln('MSR1: Marcar ilegal');
            MSR1_ComenzarIlegal:
                writeln('MSR1: Comenzar ilegal');
            MSR1_AvanzarIlegal:
                writeln('MSR1: Avanzar ilegal');
            MSR1_EAIlegal:
                writeln('MSR1: Acceso a EA ilegal');
            MSR1_NoExFichero:
                writeln('MSR1: No existe el fichero');
            MSR1_ErrCrearFichero:
                writeln('MSR1: Imposible crear fichero');
            MSR1_ErrLeerFichero:
                writeln('MSR1: Error al leer datos del fichero');
            MSR1_ErrEscribirFichero:
                writeln('MSR1: Error al escribir datos en el fichero');
          end;
          halt;
     end;
end;

procedure Registrar_MSR1(VAR S : msr1; c : real);
var
   p : MSR1_PNodoLista;
begin
     if S.estado = MSR1_creacion then
     begin
          if (c >= MSR1_MinValueAllow) and (c <= MSR1_MaxValueAllow)  then
          begin
               S.actual^.buf[S.pos_actual] := c;
               inc(S.pos_actual);
               if S.pos_actual > MSR1_ElemsBuffer then
               begin
                    new(p);
                    S.pos_actual  := 1;
                    S.actual^.sig := p;
                    S.actual      := p;
                    p^.sig        := NIL;
               end;
          end
          else
              CondicionError(S, MSR1_RealIlegal);
     end
     else
         CondicionError(S, MSR1_RegistroIlegal);
end;

procedure Marcar_MSR1(VAR S : msr1);
begin
     if S.estado = MSR1_creacion then
     begin
          S.actual^.buf[S.pos_actual] := MSR1_MarcaFin;
          S.estado := MSR1_marcada;
     end
     else
         CondicionError(S, MSR1_MarcaIlegal);
end;

procedure Comenzar_MSR1(VAR S : msr1);
begin
     if S.estado in [MSR1_consulta,MSR1_ninguno,MSR1_marcada] then
     begin
          S.actual := S.primero;
          S.pos_actual := 1;
          S.elem_actual := S.actual^.buf[1];
          S.estado := MSR1_consulta;
     end
     else
         CondicionError(S, MSR1_ComenzarIlegal);
end;

procedure Avanzar_MSR1(VAR S : msr1);
begin
     if (S.estado = MSR1_consulta) and (S.elem_actual <> MSR1_MarcaFin) then
     begin
          inc(S.pos_actual);
          if S.pos_actual > MSR1_ElemsBuffer then
          begin
               S.pos_actual := 1;
               S.actual := S.actual^.sig;
          end;
          S.elem_actual := S.actual^.buf[S.pos_actual];
     end
     else
         CondicionError(S, MSR1_AvanzarIlegal);
end;

function EA_MSR1(VAR S : msr1) : real;
begin
     if S.estado = MSR1_consulta
     then
         EA_MSR1 := S.elem_actual
     else
     begin
          CondicionError(S, MSR1_EAIlegal);
          EA_MSR1 := MSR1_MarcaFin;
     end;
end;

function error_MSR1(var S : msr1) : MSR1_Errores;
var
   r : MSR1_Errores;
begin
     r := S.c_error;
     S.c_error := MSR1_TodoOK;
     error_MSR1 := r;
end;

procedure Cargar_Fichero_MSR1(var S : msr1; nombre : string);
var
   f : text;
   c : real;
   input_error : boolean;

   procedure salta_lineas_blancas(var Se : msr1;
                                  var fi : text;
                                  var r_error : boolean);
   begin
        while eoln(fi) and not eof(fi) and not r_error
              and (Se.c_error = MSR1_TodoOk) do
        begin
             {$I-}
             readln(fi);
             {$I+}
             if IOResult <> 0 then
             begin
                  r_error := true;
                  CondicionError(Se, MSR1_ErrLeerFichero);
             end;
        end;
   end;

begin
     S.c_error := MSR1_TodoOk;
     assign(f, nombre);
     {$I-}
     reset(f);
     {$I+}
     if IOResult <> 0 then
          CondicionError(S, MSR1_NoExFichero)
     else
     begin
          Arrancar_MSR1(S);
          input_error := false;
          while not eof(f) and not input_error and
                (S.c_error = MSR1_TodoOk) do
          begin
               {$I-}
               readln(f, c);
               {$I+}
               if IOResult <> 0 then
               begin
                    input_error := true;
                    CondicionError(S, MSR1_ErrLeerFichero);
               end
               else
                   Registrar_MSR1(S, c);
               salta_lineas_blancas(S, f, input_error);
          end;
          Marcar_MSR1(S);
          close(f);
     end;
end;

procedure Salvar_Fichero_MSR1(VAR S : msr1; nombre : string);
var
   f : text;
   write_error : boolean;
begin
     S.c_error := MSR1_TodoOk;
     assign(f, nombre);
     {$I-}
     rewrite(f);
     {$I+}
     if IOResult <> 0 then
        CondicionError(S, MSR1_ErrCrearFichero)
     else
     begin
          Comenzar_MSR1(S);
          write_error := false;
          while (EA_MSR1(S) <> MSR1_MarcaFin) and
                not write_error and (S.c_error = MSR1_TodoOk) do
          begin
               {$I-}
               writeln(f, EA_MSR1(S));
               {$I+}
               if IOResult <> 0 then
               begin
                    write_error := true;
                    CondicionError(S, MSR1_ErrEscribirFichero);
               end
               else
                   Avanzar_MSR1(S);
          end;
          close(f);
     end;
end;

procedure HabilitarErrores_MSR1;
begin
     MSR1_ManejarErrores := true;
end;

procedure DeshabilitarErrores_MSR1;
begin
     MSR1_ManejarErrores := false;
end;

(* Rutinas de inicializacion *)

begin
     MSR1_ManejarErrores := true;
end.
