unit unitmsr2;

interface

const
     MSR2_ElemsBuffer   = 200;
     MSR2_MaxErrores    = 9;

     MSR2_TodoOK           = 0;
     MSR2_RealIlegal     = 1;
     MSR2_RegistroIlegal   = 2;
     MSR2_ComenzarIlegal   = 3;
     MSR2_AvanzarIlegal    = 4;
     MSR2_EAIlegal         = 5;
     MSR2_NoExistFichero   = 6;
     MSR2_ErrCrearFichero  = 7;
     MSR2_ErrLeerFichero   = 8;
     MSR2_ErrEscribFichero = 9;

     MSR2_MaxValueAllow  =  1000;
     MSR2_MinValueAllow  = -1000;

type
    MSR2_Errores = 0..MSR2_MaxErrores;
    MSR2_Estados = (MSR2_ninguno, MSR2_iniacc, MSR2_creacion, MSR2_consulta);

    MSR2_Indice      = 1..MSR2_ElemsBuffer;
    MSR2_Buffer      = packed array[MSR2_Indice] of real;
    MSR2_PNodoLista  = ^MSR2_NodoLista;
    MSR2_NodoLista   = record
                             buf  : MSR2_Buffer;
                             ocup : integer;
                             sig  : MSR2_PNodoLista;
                       end;

    msr2 = record
                 primero, actual : MSR2_PNodoLista;
                 pos_actual      : integer;
                 elem_actual     : real;
                 estado          : MSR2_Estados;
                 c_error         : MSR2_Errores;
           end;

procedure  TratamientoInicial_MSR2(VAR S : msr2);
function   EsVacia_MSR2(S : msr2) : boolean;
procedure  Arrancar_MSR2(VAR S : msr2);
procedure  Registrar_MSR2(VAR S : msr2; c : real);
function   EsUltimo_MSR2(S : msr2) : boolean;
procedure  InicAcceso_MSR2(VAR S : msr2);
procedure  Avanzar_MSR2(VAR S : msr2);
function   EA_MSR2(VAR S : msr2) : real;
procedure  Cargar_Fichero_MSR2(VAR S : msr2; nombre : string);
procedure  Salvar_Fichero_MSR2(VAR S : msr2; nombre : string);

function   error_MSR2(VAR S : msr2) : MSR2_Errores;
procedure  HabilitarErrores_MSR2;
procedure  DeshabilitarErrores_MSR2;

implementation

var
   MSR2_ManejarErrores : boolean;

procedure TratamientoInicial_MSR2(VAR S : msr2);
begin
     with S do
     begin
          primero    := NIL;
          actual     := NIL;
          estado     := MSR2_ninguno;
          c_error    := MSR2_TodoOK;
          pos_actual := 1;
     end;
end;

procedure CondicionError(VAR S : MSR2; err : MSR2_Errores);
begin
     S.c_error := err;
     if MSR2_ManejarErrores then
     begin
          case err of
            MSR2_RealIlegal:
                writeln('MSR2: Registrar entero ilegal');
            MSR2_RegistroIlegal:
                writeln('MSR2: Registrar no en creacion');
            MSR2_ComenzarIlegal:
                writeln('MSR2: Comenzar ilegal');
            MSR2_AvanzarIlegal:
                writeln('MSR2: Avanzar ilegal');
            MSR2_EAIlegal:
                writeln('MSR2: Acceso a EA ilegal');
            MSR2_NoExistFichero:
                writeln('MSR2: No existe el fichero');
            MSR2_ErrCrearFichero:
                writeln('MSR2: Imposible crear fichero');
            MSR2_ErrLeerFichero:
                writeln('MSR2: Error leyendo del fichero');
            MSR2_ErrEscribFichero:
                writeln('MSR2: Error escribiendo en el fichero');
          end;
          halt;
     end;
end;


function EsVacia_MSR2(S : MSR2) : boolean;
begin
     EsVacia_MSR2 := S.primero = NIL;
end;

procedure Destruir_MSR2(VAR S : MSR2);
var
   p, t : MSR2_PNodoLista;
begin
     with S do
     begin
          p       := primero;
          primero := NIL;
          actual  := NIL;
          while p <> NIL do
          begin
               t := p;
               p := p^.sig;
               dispose(t);
          end;
          estado     := MSR2_ninguno;
          c_error    := MSR2_TodoOK;
          pos_actual := 1;
     end;
end;

procedure Arrancar_MSR2(VAR S : MSR2);
begin
     Destruir_MSR2(S);
     S.estado := MSR2_creacion;
end;

procedure Registrar_MSR2(VAR S : MSR2; c : real);
var
   p : MSR2_PNodoLista;
begin
     if S.estado = MSR2_creacion then
     begin
        if (c >= MSR2_MinValueAllow) and
           (c <= MSR2_MaxValueAllow) then
        with S do
        begin
          if primero = NIL then
          begin
               new(p);
               primero    := p;
               actual     := p;
               p^.ocup    := 0;
               p^.sig     := NIL;
               pos_actual := 0;
          end;
          if actual^.ocup = MSR2_ElemsBuffer then
          begin
               new(p);
               actual^.sig := p;
               actual      := p;
               p^.ocup     := 0;
               p^.sig      := NIL;
               pos_actual  := 0;
          end;
          inc(pos_actual);
          inc(actual^.ocup);
          actual^.buf[pos_actual] := c;
        end
        else
          CondicionError(S, MSR2_RealIlegal)
     end
     else
         CondicionError(S, MSR2_RegistroIlegal);
end;

function EsUltimo_MSR2(S : MSR2) : boolean;
begin
     if EsVacia_MSR2(S) then
        EsUltimo_MSR2 := true
     else
     with S do
       EsUltimo_MSR2 := (pos_actual = actual^.ocup) and (actual^.sig = NIL);
end;

procedure InicAcceso_MSR2(VAR S : MSR2);
begin
     with S do
     begin
          actual := primero;
          pos_actual := 0;
          estado := MSR2_iniacc;
     end;
end;

procedure Avanzar_MSR2(VAR S : MSR2);
begin
     if (S.estado in [MSR2_consulta,MSR2_iniacc]) and not EsUltimo_MSR2(S)
     then
       with S do
           begin
                estado := MSR2_consulta;
                inc(pos_actual);
                if (pos_actual > MSR2_ElemsBuffer) then
                begin
                     pos_actual := 1;
                     actual := actual^.sig;
                end;
                elem_actual := actual^.buf[pos_actual];
           end
        else
           CondicionError(S, MSR2_AvanzarIlegal);
end;

function EA_MSR2(VAR S : MSR2) : real;
begin
     if S.estado = MSR2_consulta then
         EA_MSR2 := S.elem_actual
     else
     begin
          CondicionError(S, MSR2_EAIlegal);
          EA_MSR2 := 0;
     end;
end;

procedure  Cargar_Fichero_MSR2(VAR S : MSR2; nombre : string);
var
   f : text;
   read_error : boolean;
   c : real;

   procedure salta_lineas_blancas(VAR Se : MSR2;
                                  VAR fi : text;
                                  VAR err : boolean);
   begin
        while eoln(fi) and not eof(fi) and not err do
        begin
             {$I-}
             readln(fi);
             {$I+}
             if IOResult <> 0 then
             begin
                  err := true;
                  Se.c_error := MSR2_ErrLeerFichero;
             end;
        end;
   end;

begin
     assign(f, nombre);
     {$I-}
     reset(f);
     {$I+}
     if IOResult <> 0 then
        CondicionError(S, MSR2_NoExistFichero)
     else
         begin
              S.c_error := MSR2_TodoOk;
              read_error := false;
              Arrancar_MSR2(S);
              while not eof(f) and not read_error and
                    (S.c_error = MSR2_TodoOk) do
              begin
                   {$I-}
                   readln(f, c);
                   {$I+}
                   if IOResult <> 0 then
                   begin
                        read_error := true;
                        CondicionError(S, MSR2_ErrLeerFichero);
                   end
                   else
                   begin
                        Registrar_MSR2(S, c);
                        salta_lineas_blancas(S, f, read_error);
                   end;
              end;
              close(f);
         end;
end;

procedure  Salvar_Fichero_MSR2(VAR S : MSR2; nombre : string);
var
   f : text;
   write_error : boolean;
begin
     assign(f, nombre);
     {$I-}
     rewrite(f);
     {$I+}
     if IOResult <> 0 then
        CondicionError(S, MSR2_ErrCrearFichero)
     else
         begin
              S.c_error := MSR2_TodoOk;
              write_error := false;
              InicAcceso_MSR2(S);
              while not write_error and
                    (S.c_error = MSR2_TodoOk) and
                    not EsUltimo_MSR2(S) do
              begin
                   Avanzar_MSR2(S);
                   {$I-}
                   writeln(f, EA_MSR2(S));
                   {$I+}
                   if IOResult <> 0 then
                   begin
                        write_error := true;
                        CondicionError(S, MSR2_ErrEscribFichero);
                   end;
              end;
              close(f);
         end;
end;

function error_MSR2(VAR S : MSR2) : MSR2_Errores;
var
   err : MSR2_Errores;
begin
     err := S.c_error;
     S.c_error := MSR2_TodoOK;
     error_MSR2 := err;
end;

procedure HabilitarErrores_MSR2;
begin
     MSR2_ManejarErrores := true;
end;

procedure DeshabilitarErrores_MSR2;
begin
     MSR2_ManejarErrores := false;
end;


(* Rutinas de inicializacion *)
begin
     MSR2_ManejarErrores := true;
end.
